<?php

namespace Drupal\group_flex\Entity\Form;

use Drupal\group\Entity\Form\GroupForm as GroupFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for group forms.
 */
class GroupForm extends GroupFormBase {

  /**
   * The group flex settings array.
   *
   * @var array
   */
  private $groupFlexSettings;

  /**
   * The group type flex service.
   *
   * @var \Drupal\group_flex\GroupFlexGroupType
   */
  private $groupTypeFlex;

  /**
   * The group flex service.
   *
   * @var \Drupal\group_flex\GroupFlexGroup
   */
  private $groupFlex;

  /**
   * The group visibility manager.
   *
   * @var \Drupal\group_flex\Plugin\GroupVisibilityManager
   */
  private $groupVisibilityManager;

  /**
   * The flex group type saver.
   *
   * @var \Drupal\group_flex\GroupFlexGroupSaver
   */
  private $groupFlexSaver;

  /**
   * The group permissions manager.
   *
   * @var \Drupal\group_permissions\GroupPermissionsManager
   */
  private $groupPermissionsManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var static $form */
    $form = parent::create($container);
    $form->groupTypeFlex = $container->get('group_flex.group_type');
    $form->groupFlex = $container->get('group_flex.group');
    $form->groupVisibilityManager = $container->get('plugin.manager.group_visibility');
    $form->groupFlexSaver = $container->get('group_flex.group_saver');
    $form->groupPermissionsManager = $container->get('group_permission.group_permissions_manager');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    /** @var \Drupal\group\Entity\GroupInterface $group */
    $group = $this->getEntity();
    /** @var \Drupal\group\Entity\GroupTypeInterface $group_type */
    $group_type = $this->getEntity()->getGroupType();

    // The group flex logic is enabled for this group type.
    if ($this->groupTypeFlex->getFlexEnabled($group_type)) {
      $visibilityPlugins = $this->groupVisibilityManager->getAllAsArrayForGroup();
      $groupVisibility = $this->groupTypeFlex->getGroupTypeVisibility($group_type);
      $form['footer']['group_visibility'] = [
        '#title' => $this->t('Visibility'),
        '#type' => 'item',
        '#weight' => isset($form['actions']['#weight']) ? ($form['actions']['#weight'] - 1) : -1,
      ];

      // The group visibility is flexible on a group level.
      if ($this->groupTypeFlex->hasFlexibleGroupTypeVisibility($group_type)) {
        $visibilityOptions = [];
        foreach ($visibilityPlugins as $id => $pluginInstance) {
          $visibilityOptions[$id] = $pluginInstance->getGroupLabel($group_type);
        }
        if (!empty($visibilityOptions)) {
          $form['footer']['group_visibility']['#required'] = TRUE;
          $form['footer']['group_visibility']['#type'] = 'radios';
          $form['footer']['group_visibility']['#options'] = $visibilityOptions;
          $form['footer']['group_visibility']['#default_value'] = $this->groupFlex->getGroupVisibility($group);
        }
      }

      // The group type visibility cannot be changed on group level.
      if ($this->groupTypeFlex->hasFlexibleGroupTypeVisibility($group_type) === FALSE) {
        if (array_key_exists($groupVisibility, $visibilityPlugins)) {
          $pluginInstance = $visibilityPlugins[$groupVisibility];

          $visibility_explanation = $pluginInstance->getValueDescription($group_type);
          $visibility_description = $this->t('The @group_type_name visibility is @visibility_value', [
            '@group_type_name' => $group_type->label(),
            '@visibility_value' => $pluginInstance->getLabel(),
          ]);
          if ($visibility_description && $visibility_explanation) {
            $form['footer']['group_visibility']['#markup'] = '<p>' . $visibility_description . ' (' . $visibility_explanation . ')' . '</p>';
          }
        }
      }

      // The group joining method can be changed on group level.
      if ($this->groupTypeFlex->canOverrideJoiningMethod($group_type)) {
        $enabledJoiningMethodPlugins = $this->groupTypeFlex->getEnabledJoiningMethodPlugins($group_type);
        $joiningMethodOptions = [];
        foreach ($enabledJoiningMethodPlugins as $id => $pluginInstance) {
          $joiningMethodOptions[$id] = $pluginInstance->getLabel();
        }
        $form['footer']['group_joining_methods'] = [
          '#title' => $this->t('Joining methods'),
          '#type' => 'checkboxes',
          '#options' => $joiningMethodOptions,
          '#weight' => $form['footer']['group_visibility']['#weight'] + 1,
        ];
        $defaultOptions = $this->groupFlex->getDefaultJoiningMethods($group);
        $form['footer']['group_joining_methods']['#default_value'] = !empty($defaultOptions) ? $defaultOptions : array_keys($joiningMethodOptions);

        // Availability of join method depends on the group visibility.
        if (isset($visibilityOptions)) {
          /** @var \Drupal\group_flex\Plugin\GroupJoiningMethodBase $joiningMethod */
          foreach ($enabledJoiningMethodPlugins as $id => $joiningMethod) {
            if ($enabledVisibilityOptions = $joiningMethod->getVisibilityOptions()) {
              foreach ($visibilityOptions as $visibilityOptionId => $visibilityOptionLabel) {
                if (!in_array($visibilityOptionId, $enabledVisibilityOptions)) {
                  $form['footer']['group_joining_methods'][$id]['#states']['disabled'][][':input[name="group_visibility"]'] = ['value' => $visibilityOptionId];
                  $form['footer']['group_joining_methods'][$id]['#states']['unchecked'][][':input[name="group_visibility"]'] = ['value' => $visibilityOptionId];
                  $form['footer']['group_joining_methods'][$id]['#states']['unchecked'][][':input[name="group_joining_methods[' . $id . ']"]'] = ['checked' => FALSE];
                }
              }
            }
          }
        }
      }
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    // Create an array of group flex settings.
    $group_flex_settings = [
      'visibility' => $form_state->getValue('group_visibility', NULL),
      'joining_methods' => $form_state->getValue('group_joining_methods', NULL),
    ];
    $this->groupFlexSettings = $group_flex_settings;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $return = parent::save($form, $form_state);

    // Create an array of group flex settings.
    $group_flex_settings = [
      'visibility' => $form_state->getValue('group_visibility', NULL),
      'joining_methods' => $form_state->getValue('group_joining_methods', NULL),
    ];
    /** @var \Drupal\group\Entity\GroupInterface $group */
    $group = $this->entity;
    foreach ($group_flex_settings as $key => $value) {
      if ($value !== NULL) {
        if ($group && $group->id()) {
          switch ($key) {
            case 'visibility':
              $this->groupFlexSaver->saveGroupVisibility($group, $value);
              break;

            case 'joining_methods':
              $this->groupFlexSaver->saveGroupJoiningMethods($group, $value);
              break;
          }
        }
      }
    }
    return $return;
  }

  /**
   * {@inheritdoc}
   */
  public function store(array &$form, FormStateInterface $form_state) {
    parent::store($form, $form_state);
    $store = $this->privateTempStoreFactory->get('group_creator_flex');
    $store_id = $form_state->get('store_id');

    foreach ($this->groupFlexSettings as $key => $value) {
      if ($value !== NULL) {
        $store->set("$store_id:$key", $value);
      }
    }
  }

}
