<?php

namespace Drupal\group_flex;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group_permissions\GroupPermissionsManager;

/**
 * Get the group flex settings from a group.
 */
class GroupFlexGroup {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The permissions manager.
   *
   * @var \Drupal\group_permissions\GroupPermissionsManager
   */
  protected $groupPermissionsManager;

  /**
   * The flex group type helper.
   *
   * @var \Drupal\group_flex\GroupFlexGroupType
   */
  protected $flexGroupType;

  /**
   * Constructs a new GroupFlexGroup.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\group_permissions\GroupPermissionsManager $group_permission_group_permissions_manager
   *   The group permissions manager.
   * @param \Drupal\group_flex\GroupFlexGroupType $flex_group_type
   *   The group type flex.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, GroupPermissionsManager $group_permission_group_permissions_manager, GroupFlexGroupType $flex_group_type) {
    $this->entityTypeManager = $entity_type_manager;
    $this->groupPermissionsManager = $group_permission_group_permissions_manager;
    $this->flexGroupType = $flex_group_type;
  }

  /**
   * Get the group visibility for a given group.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to return the default value for.
   *
   * @return string
   *   The group visibility.
   */
  public function getGroupVisibility(GroupInterface $group) {
    // @todo retrieve the default group type permission.
    if (!$group->id()) {
      return GROUP_FLEX_TYPE_VIS_PUBLIC;
    }

    // If this is an existing group add default based on permissions.
    $group_permission = $this->groupPermissionsManager->getCustomPermissions($group);
    if (array_key_exists($group->getGroupType()->getOutsiderRoleId(), $group_permission) &&
      !in_array('view group', $group_permission[$group->getGroupType()->getOutsiderRoleId()])) {
      return GROUP_FLEX_TYPE_VIS_PRIVATE;
    }

    return GROUP_FLEX_TYPE_VIS_PUBLIC;
  }

  /**
   * Retrieve the default joining methods for a given group.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to retrieve the joining methods for.
   *
   * @return array
   *   The default joining methods for the given group.
   */
  public function getDefaultJoiningMethods(GroupInterface $group) {
    $defaultJoiningMethods = [];

    // If this is an existing group add default based on permissions.
    if ($group->id()) {
      $joiningMethodPlugins = $this->flexGroupType->getEnabledJoiningMethodPlugins($group->getGroupType());

      $group_permission = $this->groupPermissionsManager->getCustomPermissions($group);
      foreach ($joiningMethodPlugins as $pluginId => $pluginInstance) {
        $pluginPermissions = $pluginInstance->getGroupPermissions($group);
        if (!empty($pluginPermissions)) {
          foreach ($pluginPermissions as $roleId => $rolePermissions) {
            if (array_key_exists($roleId, $group_permission)) {
              foreach ($rolePermissions as $rolePermission) {
                if (in_array($rolePermission, $group_permission[$roleId])) {
                  $defaultJoiningMethods[] = $pluginId;
                }
              }
            }
          }
        }
      }
    }
    return $defaultJoiningMethods;
  }

}
