<?php

namespace Drupal\group_flex;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupInterface;
use Drupal\group\Entity\GroupTypeInterface;
use Drupal\group_flex\Plugin\GroupJoiningMethodManager;
use Drupal\group_flex\Plugin\GroupVisibilityManager;
use Drupal\group_permissions\Entity\GroupPermission;
use Drupal\group_permissions\GroupPermissionsManager;

/**
 * Saving of a Group to implement the correct group type permissions.
 */
class GroupFlexGroupSaver {

  /**
   * Drupal\Core\Entity\EntityTypeManagerInterface definition.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Drupal\group_permissions\GroupPermissionsManager definition.
   *
   * @var \Drupal\group_permissions\GroupPermissionsManager
   */
  protected $groupPermissionGroupPermissionsManager;

  /**
   * The group visibility manager.
   *
   * @var \Drupal\group_flex\Plugin\GroupVisibilityManager
   */
  private $groupVisibilityManager;

  /**
   * The group joining method manager.
   *
   * @var \Drupal\group_flex\Plugin\GroupJoiningMethodManager
   */
  private $groupJoiningMethodManager;

  /**
   * Constructs a new GroupFlexGroupSaver object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\group_permissions\GroupPermissionsManager $group_permission_group_permissions_manager
   *   The group permissions manager.
   * @param \Drupal\group_flex\Plugin\GroupVisibilityManager $groupVisibilityManager
   *   The group visibility manager.
   * @param \Drupal\group_flex\Plugin\GroupJoiningMethodManager $groupJoiningMethodManager
   *   The group joining method manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, GroupPermissionsManager $group_permission_group_permissions_manager, GroupVisibilityManager $groupVisibilityManager, GroupJoiningMethodManager $groupJoiningMethodManager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->groupPermissionGroupPermissionsManager = $group_permission_group_permissions_manager;
    $this->groupVisibilityManager = $groupVisibilityManager;
    $this->groupJoiningMethodManager = $groupJoiningMethodManager;
  }

  /**
   * Save the group visibility.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to save.
   * @param string $group_visibility
   *   The desired visibility of the group.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function saveGroupVisibility(GroupInterface $group, string $group_visibility) {
    $group_permission = $this->getGroupPermissionObject($group);

    if ($group_permission) {
      $permissions = $group_permission->getPermissions();
      /** @var \Drupal\group_flex\Plugin\GroupVisibilityBase $pluginInstance */
      foreach ($this->getAllGroupVisibility() as $id => $pluginInstance) {
        // Retrieve the current group visibility plugin.
        if ($group_visibility == $id) {
          $pluginPermissions = $pluginInstance->getGroupPermissions($group);
          foreach ($pluginPermissions as $role => $rolePermissions) {
            foreach ($rolePermissions as $permission) {
              if (!array_key_exists($role, $permissions) || !in_array($permission, $permissions[$role])) {
                $permissions[$role][] = $permission;
              }
            }
          }
          if (!empty($pluginInstance->getDisallowedGroupPermissions($group))) {
            foreach ($pluginInstance->getDisallowedGroupPermissions($group) as $role => $rolePermissions) {
              foreach ($rolePermissions as $permission) {
                if (array_key_exists($role, $permissions) || in_array($permission, $permissions[$role])) {
                  $permissions[$role] = array_diff($permissions[$role], [$permission]);
                }
              }
            }
          }
        }
      }
      $group_permission->setPermissions($permissions);

      $violations = $group_permission->validate();
      if (count($violations) == 0) {
        $group_permission->save();
      }
      else {
        $message = '';
        foreach ($violations as $violation) {
          $message .= "\n" . $violation->getMessage();
        }
        throw new \Exception('Group permissions are not saved correctly, because:' . $message);
      }
    }

    // Save the group entity to reset the cache tags.
    $group->save();
  }

  /**
   * Save the group joining methods.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to save.
   * @param array $group_joining_methods
   *   The desired joining methods of the group.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function saveGroupJoiningMethods(GroupInterface $group, array $group_joining_methods) {
    $group_permission = $this->getGroupPermissionObject($group);

    if ($group_permission) {
      $permissions = $group_permission->getPermissions();
      /** @var \Drupal\group_flex\Plugin\GroupJoiningMethodBase $pluginInstance */
      foreach ($this->getAllJoiningMethods() as $id => $pluginInstance) {
        // Retrieve the current group joining method plugin.
        if (in_array($id, $group_joining_methods) && $group_joining_methods[$id] === $id) {
          $pluginPermissions = $pluginInstance->getGroupPermissions($group);
          foreach ($pluginPermissions as $role => $rolePermissions) {
            foreach ($rolePermissions as $permission) {
              if (!array_key_exists($role, $permissions) || !in_array($permission, $permissions[$role])) {
                $permissions[$role][] = $permission;
              }
            }
          }
        }
        else {
          if (!empty($pluginInstance->getDisallowedGroupPermissions($group))) {
            foreach ($pluginInstance->getDisallowedGroupPermissions($group) as $role => $rolePermissions) {
              foreach ($rolePermissions as $permission) {
                if (array_key_exists($role, $permissions) || in_array($permission, $permissions[$role])) {
                  $permissions[$role] = array_diff($permissions[$role], [$permission]);
                }
              }
            }
          }
        }
      }
      $group_permission->setPermissions($permissions);

      $violations = $group_permission->validate();
      if (count($violations) == 0) {
        $group_permission->save();
      }
      else {
        throw new \Exception('Group permissions are not saved correctly.');
      }
    }

    // Save the group entity to reset the cache tags.
    $group->save();
  }

  /**
   * Get the default permissions for the given group type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The group type to return the permissions for.
   *
   * @return array
   *   An array of permissions keyed by role.
   */
  public function getDefaultGroupTypePermissions(GroupTypeInterface $groupType) {
    $permissions = [];

    /** @var \Drupal\group\Entity\GroupRoleInterface $groupRole */
    foreach ($groupType->getRoles() as $groupRoleId => $groupRole) {
      $permissions[$groupRoleId] = $groupRole->getPermissions();
    }

    return $permissions;
  }

  /**
   * Get all joining methods.
   *
   * @return array
   *   An array of joining methods containing the PluginInstances.
   */
  public function getAllJoiningMethods() {
    return $this->groupJoiningMethodManager->getAllAsArray();
  }

  /**
   * Get all group visibility.
   *
   * @return array
   *   An array of group visibilities containing the PluginInstances.
   */
  public function getAllGroupVisibility() {
    return $this->groupVisibilityManager->getAllAsArray();
  }

  /**
   * Get the groupPermission object, will create a new one if needed.
   *
   * @param \Drupal\group\Entity\GroupInterface $group
   *   The group to get the group permission object for.
   *
   * @return \Drupal\group_permissions\Entity\GroupPermission|null
   *   The (new) group permission object, returns NULL if something went wrong.
   */
  private function getGroupPermissionObject(GroupInterface $group) {
    $groupPermission = NULL;
    /** @var \Drupal\group_permissions\Entity\GroupPermission $groupPermission */
    if (!$group->isNew()) {
      $groupPermission = $this->groupPermissionGroupPermissionsManager->getGroupPermission($group);
    }
    if ($groupPermission === NULL) {
      // Create the entity.
      $groupPermission = GroupPermission::create([
        'gid' => $group->id(),
        'permissions' => $this->getDefaultGroupTypePermissions($group->getGroupType()),
      ]);
    }
    return $groupPermission;
  }

}
