<?php

namespace Drupal\group_flex;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupTypeInterface;
use Drupal\group_flex\Plugin\GroupJoiningMethodManager;

/**
 * Get the group flex settings from a group type.
 */
class GroupFlexGroupType {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The group joining method manager.
   *
   * @var \Drupal\group_flex\Plugin\GroupJoiningMethodManager
   */
  private $groupJoiningMethodManager;

  /**
   * Constructs a new GroupFlexGroupType.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\group_flex\Plugin\GroupJoiningMethodManager $group_joining_method_manager
   *   The group joining method manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, GroupJoiningMethodManager $group_joining_method_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->groupJoiningMethodManager = $group_joining_method_manager;
  }

  /**
   * Get whether flex is enabled for the group type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The Group Type.
   *
   * @return bool
   *   Returns TRUE if group_flex_enabler is TRUE.
   */
  public function getFlexEnabled(GroupTypeInterface $groupType) {
    return $groupType->getThirdPartySetting('group_flex', 'group_flex_enabler', FALSE);
  }

  /**
   * Get the group type visibility.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The Group Type.
   *
   * @return string|null
   *   Returns the visibility of the group.
   */
  public function getGroupTypeVisibility(GroupTypeInterface $groupType) {
    return $groupType->getThirdPartySetting('group_flex', 'group_type_visibility', NULL);
  }

  /**
   * Returns TRUE if the group type visibility can be changed on group level.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The Group Type.
   *
   * @return bool
   *   Returns TRUE if the group type visibility is flexible.
   */
  public function hasFlexibleGroupTypeVisibility(GroupTypeInterface $groupType) {
    return $this->getGroupTypeVisibility($groupType) === GROUP_FLEX_TYPE_VIS_FLEX;
  }

  /**
   * Get the value from the group type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The Group Type.
   *
   * @return array
   *   Returns an array of group type joining methods.
   */
  public function getJoiningMethods(GroupTypeInterface $groupType) {
    return $groupType->getThirdPartySetting('group_flex', 'group_type_joining_method', []);
  }

  /**
   * Whether the creator can override the joining method on the group level.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The Group Type.
   *
   * @return bool
   *   Returns TRUE if creator can override joining method.
   */
  public function canOverrideJoiningMethod(GroupTypeInterface $groupType) {
    return $groupType->getThirdPartySetting('group_flex', 'group_type_joining_method_override', FALSE);
  }

  /**
   * Get the enabled joining method plugins for given group type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $group_type
   *   The Group Type.
   *
   * @return array
   *   The array of enabled joining method plugins.
   */
  public function getEnabledJoiningMethodPlugins(GroupTypeInterface $group_type) {
    $enabled_joining_methods = $this->getJoiningMethods($group_type);
    $joiningMethodPlugins = $this->groupJoiningMethodManager->getAllAsArray();

    foreach ($joiningMethodPlugins as $id => $pluginInstance) {
      if (!in_array($id, $enabled_joining_methods)) {
        // Unset the value when it is not in the enabled joining methods.
        unset($joiningMethodPlugins[$id]);
      }
    }
    return $joiningMethodPlugins;
  }

}
