<?php

namespace Drupal\group_flex;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\group\Entity\GroupTypeInterface;
use Drupal\group_flex\Plugin\GroupJoiningMethodManager;
use Drupal\group_flex\Plugin\GroupVisibilityManager;

/**
 * Saves a group flex settings form in the group type interface.
 */
class GroupFlexGroupTypeSaver {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The group visibility manager.
   *
   * @var \Drupal\group_flex\Plugin\GroupVisibilityManager
   */
  private $groupVisibilityManager;

  /**
   * The group joining method manager.
   *
   * @var \Drupal\group_flex\Plugin\GroupJoiningMethodManager
   */
  private $groupJoiningMethodManager;

  /**
   * Constructs a new GroupFlexGroupTypeSaver.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\group_flex\Plugin\GroupVisibilityManager $groupVisibilityManager
   *   The group visibility manager.
   * @param \Drupal\group_flex\Plugin\GroupJoiningMethodManager $groupJoiningMethodManager
   *   The group joining method manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, GroupVisibilityManager $groupVisibilityManager, GroupJoiningMethodManager $groupJoiningMethodManager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->groupVisibilityManager = $groupVisibilityManager;
    $this->groupJoiningMethodManager = $groupJoiningMethodManager;
  }

  /**
   * Save the group flex settings.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The Group Type.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function save(GroupTypeInterface $groupType) {
    $this->saveGroupVisibility($groupType);
    $this->saveJoiningMethods($groupType);
  }

  /**
   * Save the group visibility settings.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The group type.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  protected function saveGroupVisibility(GroupTypeInterface $groupType) {
    $visibility = $groupType->getThirdPartySetting('group_flex', 'group_type_visibility', NULL);
    /** @var \Drupal\group_flex\Plugin\GroupVisibilityBase $pluginInstance */
    $groupVisibilities = $this->getAllGroupVisibility();
    foreach ($groupVisibilities as $id => $pluginInstance) {
      if ($visibility !== $id) {
        $pluginInstance->disableGroupType($groupType);
      }
    }
    if (isset($groupVisibilities[$visibility])) {
      $groupVisibilities[$visibility]->enableGroupType($groupType);
    }
  }

  /**
   * Saves the joining methods for the given group type.
   *
   * @param \Drupal\group\Entity\GroupTypeInterface $groupType
   *   The group type.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function saveJoiningMethods(GroupTypeInterface $groupType) {
    $thirdPartySettingsJoiningMethods = $groupType->getThirdPartySetting('group_flex', 'group_type_joining_method', []);
    /** @var \Drupal\group_flex\Plugin\GroupJoiningMethodBase $pluginInstance */
    $joiningMethods = $this->getAllJoiningMethods();
    foreach ($joiningMethods as $id => $pluginInstance) {
      $status = isset($thirdPartySettingsJoiningMethods[$id]) ? $thirdPartySettingsJoiningMethods[$id] : 0;
      // @todo Make sure enable is run after the disable.
      if ($status === $id) {
        $pluginInstance->enableGroupType($groupType);
      }
      else {
        $pluginInstance->disableGroupType($groupType);
      }
    }
  }

  /**
   * Get all joining methods.
   *
   * @return array
   *   An array of joining methods containing the PluginInstances.
   */
  public function getAllJoiningMethods() {
    return $this->groupJoiningMethodManager->getAllAsArray();
  }

  /**
   * Get all group visibility.
   *
   * @return array
   *   An array of group visibilities containing the PluginInstances.
   */
  public function getAllGroupVisibility() {
    return $this->groupVisibilityManager->getAllAsArray();
  }

}
